@@ -1,19 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+use std::{mem, ptr};
+
+use glib::{prelude::*, translate::*};
+
 use crate::AudioEncoder;
-use glib::prelude::*;
-use glib::translate::*;
-use std::mem;
-use std::ptr;
 
 pub trait AudioEncoderExtManual: 'static {
-    #[doc(alias = "gst_audio_encoder_finish_frame")]
-    fn finish_frame(
-        &self,
-        buffer: Option<gst::Buffer>,
-        frames: i32,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
     #[doc(alias = "gst_audio_encoder_negotiate")]
     fn negotiate(&self) -> Result<(), gst::FlowError>;
 
@@ -24,26 +17,15 @@ pub trait AudioEncoderExtManual: 'static {
     #[doc(alias = "gst_audio_encoder_get_allocator")]
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams);
 
+    #[doc(alias = "gst_audio_encoder_set_headers")]
+    fn set_headers(&self, headers: impl IntoIterator<Item = gst::Buffer>);
+
     fn sink_pad(&self) -> &gst::Pad;
 
     fn src_pad(&self) -> &gst::Pad;
 }
 
 impl<O: IsA<AudioEncoder>> AudioEncoderExtManual for O {
-    fn finish_frame(
-        &self,
-        buffer: Option<gst::Buffer>,
-        frames: i32,
-    ) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_audio_encoder_finish_frame(
-                self.as_ref().to_glib_none().0,
-                buffer.map(|b| b.into_glib_ptr()).unwrap_or(ptr::null_mut()),
-                frames,
-            ))
-        }
-    }
-
     fn negotiate(&self) -> Result<(), gst::FlowError> {
         unsafe {
             let ret = from_glib(ffi::gst_audio_encoder_negotiate(
@@ -74,13 +56,25 @@ impl<O: IsA<AudioEncoder>> AudioEncoderExtManual for O {
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams) {
         unsafe {
             let mut allocator = ptr::null_mut();
-            let mut params = mem::zeroed();
+            let mut params = mem::MaybeUninit::uninit();
             ffi::gst_audio_encoder_get_allocator(
                 self.as_ref().to_glib_none().0,
                 &mut allocator,
-                &mut params,
+                params.as_mut_ptr(),
+            );
+            (from_glib_full(allocator), params.assume_init().into())
+        }
+    }
+
+    fn set_headers(&self, headers: impl IntoIterator<Item = gst::Buffer>) {
+        unsafe {
+            ffi::gst_audio_encoder_set_headers(
+                self.as_ref().to_glib_none().0,
+                headers
+                    .into_iter()
+                    .collect::<glib::List<_>>()
+                    .into_glib_ptr(),
             );
-            (from_glib_full(allocator), params.into())
         }
     }
 
